﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using Microsoft.Research.DynamicDataDisplay;
using Microsoft.Research.DynamicDataDisplay.DataSources;
using System.Globalization;
using System.Reflection;
using System.IO;
using System.IO.Ports;
using Microsoft.Research.DynamicDataDisplay.Charts.Navigation;
using Microsoft.Win32;

namespace IVCurveTracer
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {

        // Three observable data sources. Observable data source contains
        // inside ObservableCollection. Modification of collection instantly modify
        // visual representation of graph. 
        ObservableDataSource<Point> source1 = null;
        ObservableDataSource<Point> source2 = null;
        ObservableDataSource<Point> source3 = null;
        ObservableDataSource<Point> source4 = null;
        ObservableDataSource<Point> source5 = null;
        StringComparer stringComparer = StringComparer.OrdinalIgnoreCase;
        SerialPort sp = new SerialPort();
        





        int program_state = 1;
        int current_data_series = 1;
        int[] N = new int[6];
        Single[,] V_array = new Single[6,500];
        Single[,] C_array = new Single[6,500];
        Single[] Pmax = new Single[6];
        Single[] VPmax = new Single[6];
        Single[] CPmax = new Single[6];
        Single[] Voc = new Single[6];
        Single[] Isc = new Single[6];

        Single Pintegral = 0;

        //Name for program states, used for status bar
        String[] state_name = new String[7] { "", "", "", "Ready", "Running", "Analyse", "Cool Down" };
        




        public MainWindow()
        {
            InitializeComponent();
            this.Loaded += Window_Loaded;
        }

        
        private void Window_Loaded(object sender, RoutedEventArgs e)
        {

            
            // Start computation process in second thread
            Thread simThread = new Thread(new ThreadStart(Simulation));
            simThread.IsBackground = true;
            simThread.Start();
            
            
        }
        private void Simulation()
        {
           
            while (true)
            {
                try
                {

                    //update status bar
                    programstatedisplay.Dispatcher.Invoke(() => { programstatedisplay.Text = state_name[program_state]; });
                }
                catch (Exception) { }
                
                switch (program_state)
                {
                    case 1:
                        //not connected
                        Thread.Sleep(10);
                        break;

                    

                    case 2:
                        //Waiting for device
                        sp.WriteLine("C");
                        try
                        {
                            
                            Thread.Sleep(10);
                            string message1 = sp.ReadLine();
                            if (stringComparer.Equals("A", message1))
                            {
                                Connect_Text.Dispatcher.Invoke(() => { Connect_Text.Text = "Connected"; });
                                //enable capture button
                                run_button.Dispatcher.Invoke(() => { run_button.IsEnabled = true; ; });
                                //begin waiting for series of data
                                program_state = 3;
                            }
                            else
                            {
                                
                            }
                        }
                        catch (Exception) { }

                        break;

                    case 3:
                        //Waiting to start series
                        //if start series move to state 4
                        try
                        {


                            string message2 = sp.ReadLine();
                            if (stringComparer.Equals("S", message2))
                            {
                                
                                if (current_data_series == 1)
                                {
                                    // Create first source

                                    source1 = new ObservableDataSource<Point>();
                                    // Set identity mapping of point in collection to point on plot
                                    source1.SetXYMapping(p => p);



                                    
                                    // plotter.AddLineGraph(source1, 2, "Curve 1");
                                    plotter.Dispatcher.Invoke(() => { plotter.AddLineGraph(source1, Color.FromRgb(255, 25, 25), 2, "Curve 1"); });
                                }
                                else if (current_data_series == 2)
                                {
                                    // Create second source

                                    source2 = new ObservableDataSource<Point>();
                                    // Set identity mapping of point in collection to point on plot
                                    source2.SetXYMapping(p => p);



                                    
                                    // plotter.AddLineGraph(source1, 2, "Curve 1");
                                    plotter.Dispatcher.Invoke(() => { plotter.AddLineGraph(source2, Color.FromRgb(97, 255, 110), 2, "Curve 2"); });
                                }
                                else if (current_data_series == 3)
                                {
                                    // Create third source

                                    source3 = new ObservableDataSource<Point>();
                                    // Set identity mapping of point in collection to point on plot
                                    source3.SetXYMapping(p => p);



                                    
                                    // plotter.AddLineGraph(source1, 2, "Curve 3");
                                    plotter.Dispatcher.Invoke(() => { plotter.AddLineGraph(source3, Color.FromRgb(0, 38, 255), 2, "Curve 3"); });
                                }
                                else if (current_data_series == 4)
                                {
                                    // Create forth source

                                    source4 = new ObservableDataSource<Point>();
                                    // Set identity mapping of point in collection to point on plot
                                    source4.SetXYMapping(p => p);




                                    // plotter.AddLineGraph(source1, 2, "Curve 3");
                                    plotter.Dispatcher.Invoke(() => { plotter.AddLineGraph(source4, Color.FromRgb(203, 9, 168), 2, "Curve 4"); });
                                }
                                else if (current_data_series == 5)
                                {
                                    // Create fifth source

                                    source5 = new ObservableDataSource<Point>();
                                    // Set identity mapping of point in collection to point on plot
                                    source5.SetXYMapping(p => p);




                                    // plotter.AddLineGraph(source1, 2, "Curve 3");
                                    plotter.Dispatcher.Invoke(() => { plotter.AddLineGraph(source5, Color.FromRgb(255, 151, 0), 2, "Curve 5"); });
                                }
                                program_state = 4;
                                N[current_data_series] = 0;
                                Pintegral = 0;
                            }
                        }
                        catch (Exception) { }
                        
                        
                        break;

                    case 4:
                        //receive series
                        //if end of series move to state 5
                        float V = 0;
                        float  C = 0;

                        //Add try to prevent exception during disconnect
                        string message3 = sp.ReadLine();
                        //if end of series, analyse data
                        if (stringComparer.Equals("X", message3))
                        {
                            program_state = 5;
                        }
                        //if overcurrent is detected, display error and analyse existing data
                        else if (stringComparer.Equals("E", message3))
                        {
                            MessageBox.Show("Overcurrent detected, test aborted!");
                            program_state = 6;
                        }
                        else if (stringComparer.Equals("V", message3.Substring(0,1)))
                        {
                            
                            V = Single.Parse(message3.Substring(1, 4));
                            C = Single.Parse(message3.Substring(5, 4));
                            V = (V - 1000) / 100;
                            //add data to array for analysis later
                            V_array[current_data_series,N[current_data_series]] = V;
                            C = (C - 1000) / 100;
                            C_array[current_data_series,N[current_data_series]] = C;
                            N[current_data_series]++;

                            //compute Power integral for cooldown period
                            if (C < 0)
                            {
                                Pintegral += -C * (V + 12) * 20;
                            }
                            else
                            {
                                Pintegral += C * (12 - V) * 20;
                            }
                            
                            Point next = new Point(V, C);

                            //Add point to plot
                            if (current_data_series == 1)
                            {
                                source1.AppendAsync(Dispatcher, next);
                            }
                            else if (current_data_series == 2)
                            {
                                source2.AppendAsync(Dispatcher, next);
                            }
                            else if (current_data_series == 3)
                            {
                                source3.AppendAsync(Dispatcher, next);
                            }
                            else if (current_data_series == 4)
                            {
                                source4.AppendAsync(Dispatcher, next);
                            }
                            else if (current_data_series == 5)
                            {
                                source5.AppendAsync(Dispatcher, next);
                            }
                        }
                        
                        break;

                    case 5:
                        //analyse data

                        
                        Pmax[current_data_series] = 0;
                        int maxI = 0;
                        
                        Voc[current_data_series] = 0;
                        Single Coc = 100;
                        Isc[current_data_series] = 0;
                        Single Vsc = 100;


                        int i = 0;
                        while (i<N[current_data_series])
                        {
                            
                            if(V_array[current_data_series,i]*C_array[current_data_series,i] < Pmax[current_data_series])
                            {
                                Pmax[current_data_series] = V_array[current_data_series,i] * C_array[current_data_series,i];
                                maxI = i;
                                
                            }
                            if(Math.Abs(C_array[current_data_series,i])< Coc)
                            {
                                Voc[current_data_series] = V_array[current_data_series,i];
                                Coc = Math.Abs(C_array[current_data_series,i]);
                            }
                            if (Math.Abs(V_array[current_data_series,i]) < Vsc)
                            {
                                Vsc = Math.Abs(V_array[current_data_series,i]);
                                Isc[current_data_series] = C_array[current_data_series,i];
                            }
                            i++;

                        }

                        //If Voc or Isc is never found, set as not a number
                        if(Coc>0.2)
                        {
                            Voc[current_data_series] = float.NaN;
                        }
                        if(Vsc>0.2)
                        {
                            Isc[current_data_series] = float.NaN;
                        }
                        //take absolute value of Pmax
                        Pmax[current_data_series] = Math.Abs(Pmax[current_data_series]);
                        //Save voltage and current at Pmax for exporting
                        VPmax[current_data_series] = V_array[current_data_series, maxI];
                        CPmax[current_data_series] = C_array[current_data_series, maxI];

                        //add data to UI Stackpanel
                        switch (current_data_series)
                        {
                            case 1:

                                //Do a bunch of invokes to show analysis in UI
                                curve_1_pmax.Dispatcher.Invoke(() => { curve_1_pmax.Text="Pmax = "+ Pmax[current_data_series].ToString("N") + " W"; });
                                curve_1_vpmax.Dispatcher.Invoke(() => { curve_1_vpmax.Text = "V@Pmax = " + V_array[current_data_series,maxI].ToString("N") + " V"; });
                                curve_1_ipmax.Dispatcher.Invoke(() => { curve_1_ipmax.Text = "I@Pmax = " + C_array[current_data_series,maxI].ToString("N") + " A"; });
                                curve_1_voc.Dispatcher.Invoke(() => { curve_1_voc.Text = "Voc = " + Voc[current_data_series].ToString("N") + " V"; });
                                curve_1_isc.Dispatcher.Invoke(() => { curve_1_isc.Text = "Isc = " + Isc[current_data_series].ToString("N") + " A"; });
                                //make the analysis visible
                                curve_1_stack.Dispatcher.Invoke(() => { curve_1_stack.Visibility = Visibility.Visible; });
                                break;

                            case 2:
                                //Do a bunch of invokes to show analysis in UI
                                curve_2_pmax.Dispatcher.Invoke(() => { curve_2_pmax.Text = "Pmax = " + Pmax[current_data_series].ToString("N") + " W"; });
                                curve_2_vpmax.Dispatcher.Invoke(() => { curve_2_vpmax.Text = "V@Pmax = " + V_array[current_data_series,maxI].ToString("N") + " V"; });
                                curve_2_ipmax.Dispatcher.Invoke(() => { curve_2_ipmax.Text = "I@Pmax = " + C_array[current_data_series,maxI].ToString("N") + " A"; });
                                curve_2_voc.Dispatcher.Invoke(() => { curve_2_voc.Text = "Voc = " + Voc[current_data_series].ToString("N") + " V"; });
                                curve_2_isc.Dispatcher.Invoke(() => { curve_2_isc.Text = "Isc = " + Isc[current_data_series].ToString("N") + " A"; });
                                //make the analysis visible
                                curve_2_stack.Dispatcher.Invoke(() => { curve_2_stack.Visibility = Visibility.Visible; });
                                break;

                            case 3:
                                //Do a bunch of invokes to show analysis in UI
                                curve_3_pmax.Dispatcher.Invoke(() => { curve_3_pmax.Text = "Pmax = " + Pmax[current_data_series].ToString("N") + " W"; });
                                curve_3_vpmax.Dispatcher.Invoke(() => { curve_3_vpmax.Text = "V@Pmax = " + V_array[current_data_series,maxI].ToString("N") + " V"; });
                                curve_3_ipmax.Dispatcher.Invoke(() => { curve_3_ipmax.Text = "I@Pmax = " + C_array[current_data_series,maxI].ToString("N") + " A"; });
                                curve_3_voc.Dispatcher.Invoke(() => { curve_3_voc.Text = "Voc = " + Voc[current_data_series].ToString("N") + " V"; });
                                curve_3_isc.Dispatcher.Invoke(() => { curve_3_isc.Text = "Isc = " + Isc[current_data_series].ToString("N") + " A"; });
                                //make the analysis visible
                                curve_3_stack.Dispatcher.Invoke(() => { curve_3_stack.Visibility = Visibility.Visible; });
                                break;

                            case 4:
                                //Do a bunch of invokes to show analysis in UI
                                curve_4_pmax.Dispatcher.Invoke(() => { curve_4_pmax.Text = "Pmax = " + Pmax[current_data_series].ToString("N") + " W"; });
                                curve_4_vpmax.Dispatcher.Invoke(() => { curve_4_vpmax.Text = "V@Pmax = " + V_array[current_data_series,maxI].ToString("N") + " V"; });
                                curve_4_ipmax.Dispatcher.Invoke(() => { curve_4_ipmax.Text = "I@Pmax = " + C_array[current_data_series,maxI].ToString("N") + " A"; });
                                curve_4_voc.Dispatcher.Invoke(() => { curve_4_voc.Text = "Voc = " + Voc[current_data_series].ToString("N") + " V"; });
                                curve_4_isc.Dispatcher.Invoke(() => { curve_4_isc.Text = "Isc = " + Isc[current_data_series].ToString("N") + " A"; });
                                //make the analysis visible
                                curve_4_stack.Dispatcher.Invoke(() => { curve_4_stack.Visibility = Visibility.Visible; });
                                break;

                            case 5:
                                //Do a bunch of invokes to show analysis in UI
                                curve_5_pmax.Dispatcher.Invoke(() => { curve_5_pmax.Text = "Pmax = " + Pmax[current_data_series].ToString("N") + " W"; });
                                curve_5_vpmax.Dispatcher.Invoke(() => { curve_5_vpmax.Text = "V@Pmax = " + V_array[current_data_series,maxI].ToString("N") + " V"; });
                                curve_5_ipmax.Dispatcher.Invoke(() => { curve_5_ipmax.Text = "I@Pmax = " + C_array[current_data_series,maxI].ToString("N") + " A"; });
                                curve_5_voc.Dispatcher.Invoke(() => { curve_5_voc.Text = "Voc = " + Voc[current_data_series].ToString("N") + " V"; });
                                curve_5_isc.Dispatcher.Invoke(() => { curve_5_isc.Text = "Isc = " + Isc[current_data_series].ToString("N") + " A"; });
                                //make the analysis visible
                                curve_5_stack.Dispatcher.Invoke(() => { curve_5_stack.Visibility = Visibility.Visible; });
                                break;

                        }

                        
                       
                        
                        //move to cooldown
                        program_state = 6;

                        break;

                    case 6:
                        //cooldown
                        //If the average power disipated by the output op-amp is greater then 50 watts, sleep to allow the output op-amp to cool
                        int T = 0;
                        if ((Pintegral / (N[current_data_series] * 20)) > 50)
                        {


                            T = Convert.ToInt32((Pintegral / 50) - N[current_data_series] * 20);
                        }
                        Thread.Sleep(T);
                        
                        current_data_series++;
                        program_state = 3;

                        //enable capture button
                        run_button.Dispatcher.Invoke(() => { run_button.IsEnabled = true; });
                        break;


                    
                }
                
                
                Thread.Sleep(5);

            }
        }
        

        private void Export_Click(object sender, RoutedEventArgs e)
        {
            //Only allow export when not plotting results
            if (program_state < 4 && current_data_series > 1)
            {
                SaveFileDialog dlg = new SaveFileDialog();
                dlg.FileName = "IVCurves"; // Default file name
                dlg.DefaultExt = ".csv"; // Default file extension
                dlg.Filter = "Comma Seperated Values (.csv)|*.csv"; // Filter files by extension

                // Show save file dialog box
                Nullable<bool> result = dlg.ShowDialog();

                // Process save file dialog box results
                if (result == true)
                {

                    // Save document
                    string filename = dlg.FileName;

                    using (StreamWriter sw = new StreamWriter(filename))
                    {
                        sw.Write("Date:,");
                        sw.WriteLine(DateTime.Now);
                        String[] curve_names = new String[6] {"", "Curve 1", "Curve 2", "Curve 3", "Curve 4", "Curve 5" };
                        int curve_count = 1;
                        while (curve_count < current_data_series)
                        {
                            sw.WriteLine();
                            sw.WriteLine(curve_names[curve_count]);
                            sw.Write("Pmax:,");
                            sw.WriteLine(Pmax[curve_count]);
                            sw.Write("V@Pmax:,");
                            sw.WriteLine(VPmax[curve_count]);
                            sw.Write("I@Pmax:,");
                            sw.WriteLine(CPmax[curve_count]);
                            sw.Write("Voc:,");
                            sw.WriteLine(Voc[curve_count]);
                            sw.Write("Isc:,");
                            sw.WriteLine(Isc[curve_count]);
                            sw.WriteLine();
                            sw.WriteLine("Voltage,Current");
                            int data_point = 0;
                            while(data_point<N[curve_count])
                            {
                                sw.Write(V_array[curve_count, data_point]);
                                sw.Write(",");
                                sw.WriteLine(C_array[curve_count, data_point]);


                                data_point++;
                            }


                            curve_count++;
                        }


                    }

                }
            }
            else
            {
                //if current plot is not finished
                if (program_state >= 4)
                {   
                    MessageBox.Show("Please wait for the current curve to finish plotting.");
                }
                //if no plots have been made
                else
                {
                    MessageBox.Show("Please plot a curve first.");
                }
            }
        
        }

        private void About_Click(object sender, RoutedEventArgs e)
        {
            MessageBox.Show("Built as part of an ECE4901 Capstone Project Spring 2016.", "About");
        }

        private void Help_Click(object sender, RoutedEventArgs e)
        {
            //open pdf help document
            try
            {
                System.Diagnostics.Process process = new System.Diagnostics.Process();
                string path = AppDomain.CurrentDomain.BaseDirectory + @"/Help.pdf";
                Uri pdf = new Uri(path, UriKind.RelativeOrAbsolute);
                process.StartInfo.FileName = pdf.LocalPath;
                process.Start();
                process.WaitForExit();
            }
            catch (Exception)
            {
                
            }
        }

       

        

        private void Connect_Click(object sender, RoutedEventArgs e)
        {
            if (!sp.IsOpen)
            {
                try
                {
                    //disable comport control to prevent change while connected
                    comport.IsEnabled = false;
                  
                    //connect to comport
                    String portName = comport.Text;
                    sp.PortName = portName;
                    sp.BaudRate = 115200;
                    sp.Open();
                    
                    //show connection in status bar
                    Connect_Bar.Fill = new SolidColorBrush(Color.FromRgb(62, 240, 10));
                    Connect_Text.Text = "Waiting For Device";
                    //move to waiting for device
                    program_state = 2;
                    sp.WriteLine("C");





                }

                catch (Exception)
                {
                    MessageBox.Show("Error Connecting");
                    comport.IsEnabled = true;
                    Connect_Bar.Fill = new SolidColorBrush(Color.FromRgb(232, 25, 25));
                    Connect_Text.Text = "Not Connected";
                    //move to idle state
                    program_state = 1;
                }
            }
            else
            {
                MessageBox.Show("Serial Port Already Open");
            }
            
        }
        private void Disconnect_Click(object sender, RoutedEventArgs e)
        {
            try
            {
                sp.Close();
                comport.IsEnabled = true;
                run_button.IsEnabled = false;
                
                Connect_Bar.Fill = new SolidColorBrush(Color.FromRgb(232, 25, 25));
                Connect_Text.Text = "Disconnected";
                //move to idle
                program_state = 1;


            }

            catch (Exception)
            {
                MessageBox.Show("Error Disonnecting");
                
            }
        }
            
            


        private void run_button_Click(object sender, RoutedEventArgs e)
        {
            if (current_data_series < 6)
            {
                try
                {
                    sp.WriteLine("B");
                    int start_voltage = Convert.ToInt32(start_voltage_slider.Value * 10);
                    sp.WriteLine(start_voltage.ToString());
                    int end_voltage = Convert.ToInt32(end_voltage_slider.Value * 10);
                    sp.WriteLine(end_voltage.ToString());
                    sp.WriteLine(sample_size.Text);

                    run_button.IsEnabled = false;
                }
                
                catch (Exception)
                {

                }
            }
            else
            {
                MessageBox.Show("Data full, press Clear to take more readings.");
            }
        }

        

        private void clear_button_Click(object sender, RoutedEventArgs e)
        {
            plotter.Children.RemoveAll((typeof(LineGraph)));
            current_data_series = 1;
            curve_1_stack.Visibility = Visibility.Collapsed;
            curve_2_stack.Visibility = Visibility.Collapsed;
            curve_3_stack.Visibility = Visibility.Collapsed;
            curve_4_stack.Visibility = Visibility.Collapsed;
            curve_5_stack.Visibility = Visibility.Collapsed;

        }

        private void cursor_checkBox_Checked(object sender, RoutedEventArgs e)
        {
            cursorGraph.Visibility = Visibility.Visible;
        }

        private void cursor_checkBox_Unchecked(object sender, RoutedEventArgs e)
        {
            cursorGraph.Visibility = Visibility.Hidden;
        }

        private void end_voltage_slider_ValueChanged(object sender, RoutedPropertyChangedEventArgs<double> e)
        {
            end_voltage_number.Text = end_voltage_slider.Value.ToString()+" V";
            if(end_voltage_slider.Value<=(start_voltage_slider.Value+0.5))
            {
                start_voltage_slider.Value = (end_voltage_slider.Value - 0.5);
            }
        }

        private void start_voltage_slider_ValueChanged(object sender, RoutedPropertyChangedEventArgs<double> e)
        {
            start_voltage_number.Text = start_voltage_slider.Value.ToString()+" V";
            try
            {
                if (start_voltage_slider.Value >= (end_voltage_slider.Value - 0.5))
                {
                    end_voltage_slider.Value = (start_voltage_slider.Value + 0.5);
                }
            }
            catch (Exception) { }
        }

        private void data_checkBox_Checked(object sender, RoutedEventArgs e)
        {
            analysis_data.Visibility = Visibility.Visible;
        }

        private void data_checkBox_Unchecked(object sender, RoutedEventArgs e)
        {
            analysis_data.Visibility = Visibility.Hidden;
        }
    }
}
